/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.server.engine;

import java.io.InputStream;
import java.text.MessageFormat;
import java.util.Properties;
import java.util.Vector;

import javax.security.auth.login.LoginContext;

import org.apache.log4j.Logger;
import org.apache.xml.resolver.tools.CatalogResolver;
import org.apache.xmlbeans.XmlObject;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag.api.configuration.WSAG4JConfiguration;
import org.ogf.graap.wsag.api.logging.LogMessage;
import org.ogf.graap.wsag.server.api.WsagMessageContext;
import org.ogf.graap.wsag.server.persistence.EmfRegistry;
import org.ogf.graap.wsag.server.persistence.IAgreementFactoryHome;
import org.ogf.graap.wsag.server.persistence.IAgreementHome;
import org.ogf.graap.wsag.server.persistence.PersistentAgreementFactory;
import org.ogf.graap.wsag.server.persistence.impl.DatabaseAgreementHome;
import org.ogf.graap.wsag.server.persistence.impl.WSAG4JPersistenceFacade;
import org.ogf.graap.wsag4j.types.configuration.ConfigurationType;
import org.ogf.graap.wsag4j.types.configuration.WSAG4JEngineConfigurationDocument;
import org.ogf.graap.wsag4j.types.configuration.WSAG4JEngineConfigurationType;
import org.ogf.graap.wsag4j.types.configuration.WSAG4JEngineInstanceType;
import org.ogf.graap.wsag4j.types.configuration.WSRFEngineConfigurationType;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.SchedulerFactory;
import org.quartz.impl.StdSchedulerFactory;

/**
 * The WSAGEngine provides access to all agreement factories configured for this engine. The available
 * factories can be retrieved via the agreement factory home interface. The agreement factory home is
 * retrieved by the {@link #getAgreementFactoryHome()} method. Before the engine can be used it must be
 * initialized. Engine initialization is triggered by the {@link #initializeEngine(String)} method.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class WsagEngine
{

    private static final Logger LOG = Logger.getLogger( WsagEngine.class );

    private static ThreadLocal<WsagMessageContext> messageContext = null;

    private static WsagEngine engine = null;

    private ConfigurationType wsag4jConfiguration = null;

    private WSRFEngineConfigurationType wsrfConfiguration = null;

    private WSAG4JEngineConfigurationType[] engineConfigurations = null;

    private String deploymentURI = null;

    private LoginContext serverLoginContext;

    private IAgreementFactoryHome agreementFactoryHome;

    private IAgreementHome agreementHome;

    private boolean allowAnonymousAccess = false;

    /**
     * Allows to specify an alternate filename for the wsrf-engine configuration via the system properties.
     * 
     * @see System#setProperty(String, String)
     */
    public static final String WSAG4J_WSRF_ENGINE_FILE_NAME = "wsag4j.wsrf-engine.config.filename";

    /**
     * Returns the message context for the current invocation (thread).
     * 
     * @return the message context
     */
    public static WsagMessageContext getWsagMessageContext()
    {

        synchronized ( messageContext )
        {
            WsagMessageContext wsagcontext = messageContext.get();

            if ( wsagcontext == null )
            {
                wsagcontext = new WsagMessageContext();
                messageContext.set( wsagcontext );
            }

            return wsagcontext;
        }
    }

    /**
     * Sets the message context for the current invocation (thread).
     * 
     * @param context
     *            the message context
     */
    public static void setWsagMessageContext( WsagMessageContext context )
    {
        WsagEngine.messageContext.set( context );
    }

    private static synchronized WsagEngine getInstance()
    {
        if ( engine == null )
        {
            engine = new WsagEngine();
        }

        return engine;
    }

    /**
     * Initializes the WSAG4J Engine. During the initialization process the WSRF engine configuration (system
     * configuration) is parsed and the referenced WSAG4J engine configurations (factory configurations) are
     * loaded. The WSAG4J engine must be initialized before making any calls to the system. By default the
     * WSRF engine configuration location is specified by the
     * {@link WsagConstants#WSAG4J_WSRF_ENGINE_CONFIG_FILE} constant. This value can be overwritten by the
     * system property {@value #WSAG4J_WSRF_ENGINE_FILE_NAME}.
     * 
     * @param defaultGatewayURL
     *            the default gateway URL
     * 
     * @throws Exception
     *             indicates an error during engine initialization
     */
    public static void initializeEngine( String defaultGatewayURL ) throws Exception
    {
        getInstance().initialize( defaultGatewayURL );
    }

    private void initialize( String defaultGatewayURL ) throws Exception
    {

        if ( LOG.isTraceEnabled() )
        {
            LOG.trace( "WsagEngine -> initializeServer" );
        }

        //
        // set the CatalogResolver as default entity resolver for XmlBeans
        //
        System.setProperty( "xmlbean.entityResolver", CatalogResolver.class.getName() );

        messageContext = new ThreadLocal<WsagMessageContext>();

        loadWSAG4JConfiguration();

        if ( LOG.isTraceEnabled() )
        {
            LOG.trace( "AFTER: WsagEngine -> loadWSAG4JConfiguration" );
        }

        initializeGatewayURI( defaultGatewayURL );

        if ( LOG.isTraceEnabled() )
        {
            LOG.trace( "AFTER: WsagEngine -> initializeGatewayURI" );
        }

        loadEngineConfigurations();

        initializePersistenceLayer();

        if ( LOG.isTraceEnabled() )
        {
            LOG.trace( "AFTER: WsagEngine -> initializePersistenceLayer" );
        }
    }

    /**
     * Shutdown of the WSAG4J engine instance.
     * 
     * @throws Exception
     *             indicates an error during engine shutdown
     */
    public static void shutdownEngine() throws Exception
    {

        try
        {
            SchedulerFactory factory = new StdSchedulerFactory();
            Scheduler scheduler = factory.getScheduler();

            if ( scheduler.isStarted() )
            {
                scheduler.shutdown();
            }
        }
        catch ( SchedulerException e )
        {
            LOG.error( "Failed to shutdown quartz scheduler.", e );
        }

        getInstance().shutdownPersistenceLayer();

        //
        // prevent memory leak
        //
        if ( messageContext != null )
        {
            messageContext.set( null );
            messageContext = null;
        }
        engine = null;
    }

    /**
     * Loads the WSAG4J configuration.
     * 
     * 
     * @throws Exception
     */
    private void loadWSAG4JConfiguration() throws Exception
    {

        if ( LOG.isTraceEnabled() )
        {
            LOG.trace( "WsagEngine -> loadWSAG4JConfiguration" );
        }

        //
        // resolve the WSAG4J configuration file name
        //
        String filename =
            System.getProperty( WSAG4J_WSRF_ENGINE_FILE_NAME, WsagConstants.WSAG4J_WSRF_ENGINE_CONFIG_FILE );

        //
        // load the WSAG4J configuration and set it in the WSAG4JEngine
        //
        wsag4jConfiguration = WSAG4JConfiguration.findWSAG4JConfiguration( filename );
        if ( wsag4jConfiguration != null )
        {

            wsrfConfiguration = wsag4jConfiguration.getWSRFEngineConfiguration();

            //
            // If the WSRF configuration was not found, throw an exception
            //
            if ( WsagEngine.getWSRFConfiguration() == null )
            {
                String message = "WSAG4J WSRF configuration was not found.";
                throw new Exception( message );
            }

        }
        else
        {

            //
            // If the WSAG4J configuration was not found, throw an exception
            //
            String message = "WSAG4J configuration was not found.";
            throw new Exception( message );

        }
    }

    /**
     * Loads the WSAG4J engine configurations specified in the wsrf-engine.config file and initializes the
     * {link {@link #engineConfigurations} property.
     */
    private void loadEngineConfigurations()
    {
        Vector<WSAG4JEngineConfigurationType> engineConfig = new Vector<WSAG4JEngineConfigurationType>();

        WSAG4JEngineInstanceType[] instances = new WSAG4JEngineInstanceType[0];
        if ( ( wsrfConfiguration != null ) && wsrfConfiguration.isSetWSAG4JEngineInstances() )
        {
            instances = wsrfConfiguration.getWSAG4JEngineInstances().getWSAG4JEngineArray();
        }

        if ( instances == null )
        {
            String message =
                "Missing section WSAG4JEngineInstances in wsag4j engine configuration. "
                    + "No Agreement Factories were instantiated.";
            LOG.warn( message );
        }
        else
        {
            for ( int i = 0; i < instances.length; i++ )
            {
                String configFile = instances[i].getEngineConfigurationFile();

                try
                {
                    InputStream in = getClass().getResourceAsStream( configFile );

                    if ( in == null )
                    {
                        LOG.warn( LogMessage.getMessage(
                            "The wsag4j engine configuration file [{0}] was not found. Skipping this entry.",
                            configFile ) );

                        continue;
                    }

                    WSAG4JEngineConfigurationDocument engineConfiguration =
                        (WSAG4JEngineConfigurationDocument) XmlObject.Factory.parse( in );
                    engineConfig.add( engineConfiguration.getWSAG4JEngineConfiguration() );
                }
                catch ( Exception e )
                {
                    LOG.error( LogMessage.getMessage(
                        "Could not load WSAG4J engine configuration {0}. Ignoring this instance. Error: {1}",
                        configFile, e.getMessage() ) );
                }
            }
        }

        this.engineConfigurations =
            engineConfig.toArray( new WSAG4JEngineConfigurationType[engineConfig.size()] );
    }

    /**
     * Initializes the WSAG4J Gateway URL.
     * 
     * @param defaultDeploymentURI
     * @throws Exception
     */
    private void initializeGatewayURI( String defaultDeploymentURI ) throws Exception
    {

        LOG.info( "WsagEngine -> initializeGatewayURI" );

        // first we try to load the deployment uri via the system properties
        String spConfiguredURI = loadViaSystemProperties();
        if ( spConfiguredURI != null )
        {

            LOG.info( "Gateway address for this service is configured via System Properties." );
            LOG.info( LogMessage.getMessage( "WS-Resources will be deployed at URI: {0}", spConfiguredURI ) );

            deploymentURI = spConfiguredURI;
            return;
        }

        // Read the gateway URI from the configuration file.
        // This value is provided by the user/administrator.
        String gatewayAddress = wsrfConfiguration.getGatewayAddress();
        if ( ( gatewayAddress != null ) && ( !gatewayAddress.equals( "" ) ) )
        {
            Object[] filler = new Object[] { gatewayAddress };
            String message =
                MessageFormat.format(
                    "Loaded gateway url from configuration file. Resources will be deployed at [{0}]",
                    filler );
            LOG.info( message );

            deploymentURI = gatewayAddress;
            return;
        }

        // Since no gateway URI was configured in the configuration
        // file, we try to generate one.
        // We use a standard way to obtain the context path name
        // which should work across all servers. We remove the
        // trailing slash, then take what's left over, which
        // should be the context path less the preceding
        // slash such as "wsag4j"
        String noDeploymentURI = "No gateway address is configured for this service";
        LOG.warn( noDeploymentURI );

        String msgGeneratedWarning = "Try to generate gateway address for service";
        LOG.warn( msgGeneratedWarning );

        deploymentURI = defaultDeploymentURI;
    }

    private String loadViaSystemProperties()
    {
        try
        {
            InputStream in = WSAG4JConfiguration.findResource( WsagConstants.WSAG4J_CONFIG_FILE );
            Properties properties = new Properties();
            properties.load( in );

            String key =
                properties.getProperty( "org.ogf.graap.wsag.gateway.key",
                    WsagConstants.WSAG4J_GATEWAY_PROPERTY );

            return System.getProperty( key );
        }
        catch ( Exception ex )
        {
            return System.getProperty( WsagConstants.WSAG4J_GATEWAY_PROPERTY );
        }
    }

    /**
     * @return the wsag4jConfiguration
     */
    public static ConfigurationType getWSAG4JConfiguration()
    {
        return getInstance().wsag4jConfiguration;
    }

    /**
     * @return the configuration
     */
    public static WSRFEngineConfigurationType getWSRFConfiguration()
    {
        return getInstance().wsrfConfiguration;
    }

    /**
     * @return the deploymentAddress
     */
    public static String getGatewayURL()
    {
        return getInstance().deploymentURI;
    }

    /**
     * Sets the WSAG4J engine login context. This login context is used to identify a WSAG4J server instance,
     * e.g. action implementations can use this login context to sign outgoing messages.
     * 
     * @param context
     *            the login context
     */
    public static void setLoginContext( LoginContext context )
    {
        getInstance().serverLoginContext = context;
    }

    /**
     * Returns the WSAG4J engine login context. This login context is used to identify a WSAG4J server
     * instance, e.g. action implementations can use this login context to sign outgoing messages.
     * 
     * @return the login context
     */
    public static LoginContext getLoginContext()
    {
        return getInstance().serverLoginContext;
    }

    /**
     * Returns the agreement factory home
     * 
     * @return the agreement factory home
     */
    public static IAgreementFactoryHome getAgreementFactoryHome()
    {
        return getInstance().agreementFactoryHome;
    }

    /**
     * Overwrites the instance created during initialization. The WS-Layer uses this to deploy the extended WS
     * version.
     * 
     * @param agreementFactoryHome
     *            the home interface to look up the agreement factories
     */
    public static void setAgreementFactoryHome( IAgreementFactoryHome agreementFactoryHome )
    {
        getInstance().agreementFactoryHome = agreementFactoryHome;
    }

    /**
     * @return the agreement home
     * 
     * @deprecated
     */
    @Deprecated
    public static IAgreementHome getAgreementHome()
    {
        return getInstance().agreementHome;
    }

    /**
     * Overwrites the instance created during initialization. The WS-Layer uses this to deploy the extended WS
     * version.
     * 
     * @param agreementHome
     *            the home interface to look up the existing agreements
     */
    public static void setAgreementHome( IAgreementHome agreementHome )
    {
        getInstance().agreementHome = agreementHome;
    }

    /**
     * Initializes the WSAG4J persistence layer.
     */
    private void initializePersistenceLayer() throws Exception
    {

        try
        {
            LOG.info( "WsagEngine -> initialize PersistenceLayer" );

            // build and initialise agreement factory home
            agreementFactoryHome = new WSAG4JPersistenceFacade( engineConfigurations );
            agreementFactoryHome.initialize();

            // build and initialise agreement home
            agreementHome = new DatabaseAgreementHome( agreementFactoryHome );

            LOG.info( "WsagEngine -> Persistence Layer initialized" );
        }
        catch ( Exception e )
        {
            LOG.error( "WsagEngine -> failed to initialize Persistence Layer", e );
            throw new Exception( "Failed to initialize persistence layer.", e );
        }
    }

    /**
     * Shutdown the persistence layer and persist all agreements that are currently managed by the
     * implementation.
     * 
     * TODO: Save current agreement states.
     */
    private void shutdownPersistenceLayer() throws Exception
    {
        // load all agreements
        PersistentAgreementFactory[] factories = getAgreementFactoryHome().list();
        for ( int i = 0; i < factories.length; i++ )
        {
            try
            {
                final String msgDoSave = "Save agreement factory ''{0}''.";
                LOG.debug( LogMessage.getMessage( msgDoSave, factories[i].getResourceId() ) );

                factories[i].save();

                final String msgSaved = "Agreement factory ''{0}'' saved.";
                LOG.debug( LogMessage.getMessage( msgSaved, factories[i].getResourceId() ) );
            }
            catch ( Exception e )
            {
                String message = "Failed to save agreement factory ''{0}''.";
                LOG.error( MessageFormat.format( message, new Object[] { factories[i].getResourceId() } ), e );
            }
        }

        // remove references to agreement (factory) home
        agreementFactoryHome = null;
        agreementHome = null;

        // close the entity manager factory
        EmfRegistry.finalizeEmfRegistry();
    }

    /**
     * Sets if anonymous access to the server is allowed.
     * 
     * @param allowAnonymousAccess
     *            the allowAnonymousAccess to set
     */
    public static void setAllowAnonymousAccess( boolean allowAnonymousAccess )
    {
        getInstance().allowAnonymousAccess = allowAnonymousAccess;
    }

    /**
     * Sets if anonymous access to the server is allowed.
     * 
     * @return the allowAnonymousAccess
     */
    public static boolean isAllowAnonymousAccess()
    {
        return getInstance().allowAnonymousAccess;
    }
}
