/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.server.persistence.impl;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.ogf.graap.wsag.api.AgreementFactory;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag.server.api.IAgreementFactory;
import org.ogf.graap.wsag.server.api.impl.AgreementFactoryContext;
import org.ogf.graap.wsag.server.api.impl.AgreementFactoryFacade;
import org.ogf.graap.wsag.server.persistence.IAgreementFactoryHome;
import org.ogf.graap.wsag.server.persistence.PersistedResourceException;
import org.ogf.graap.wsag.server.persistence.PersistentAgreementFactory;
import org.ogf.graap.wsag4j.types.configuration.WSAG4JEngineConfigurationType;

/**
 * <p>
 * Abstract implementation of an agreement factory home. This implementation provides global
 * configuration-related functions as well as it is responsible to find and load the agreement factory
 * prototype (based on the configuration).
 * </p>
 * 
 * <p>
 * All implementations of the {@link IAgreementFactoryHome} interface should extend this abstract
 * implementation.
 * </p>
 * 
 * @author Oliver Waeldrich
 * 
 */
public abstract class AbstractWSAG4JPersistence
    implements IAgreementFactoryHome
{

    // //////////////////////////////////////////////////////////////////////////////////
    // ////// variable definition section ////////
    // //////////////////////////////////////////////////////////////////////////////////

    private static final Logger LOG = Logger.getLogger( AbstractWSAG4JPersistence.class );

    /**
     * indicates that the persistence layer has changed and a reload is required.
     */
    private boolean changed = false;

    /**
     * the wsag4j engine configuration
     */
    protected WSAG4JEngineConfigurationType wsag4jConfiguration;

    /**
     * stores the persistent factories of this layer
     */
    private final Vector<PersistentAgreementFactory> persistentFactories =
        new Vector<PersistentAgreementFactory>();

    /**
     * stores the known factories by their ids for fast factory lookup
     */
    private final Map<String, PersistentAgreementFactory> factoriesById =
        new HashMap<String, PersistentAgreementFactory>();

    // //////////////////////////////////////////////////////////////////////////////////
    // ////// abstract method declaration ////////
    // //////////////////////////////////////////////////////////////////////////////////

    /**
     * Loads the agreement factories for this persistence layer. The load method is invoked during the
     * initialization process of the persistence layer (see {@link #initialize()}) or during a
     * {@link #find(String)}, {@link #list()} or {@link #remove(String)} operation
     * 
     * 
     * @return the loaded factories
     * 
     * @throws PersistedResourceException
     *             indicates an error while loading the persistent factories
     */
    protected abstract PersistentAgreementFactory[] doLoad() throws PersistedResourceException;

    /**
     * Removes a persistent factory from the persistence layer.
     * 
     * @param factory
     *            the factory to remove
     * 
     * @return <code>true</code> if the factory was removed, otherwise <code>false</code>
     * 
     * @throws PersistedResourceException
     *             indicates an error while removing the factory
     */
    protected abstract boolean doRemove( PersistentAgreementFactory factory )
        throws PersistedResourceException;

    // //////////////////////////////////////////////////////////////////////////////////
    // ////// getter/setter methods ////////
    // //////////////////////////////////////////////////////////////////////////////////

    /**
     * @return Configuration for this agreement factory home instance.
     */
    public WSAG4JEngineConfigurationType getEngineConfiguration()
    {
        return wsag4jConfiguration;
    }

    /**
     * @param configuration
     *            The configuration used to initialize this agreement factory home instance.
     */
    public void setEngineConfiguration( WSAG4JEngineConfigurationType configuration )
    {
        this.wsag4jConfiguration = configuration;
    }

    /**
     * Indicates that the persistence layer has changed and needs to be re-initialized (see
     * {@link #initialize()}).
     * 
     * @return <code>true</code> if the persistence layer needs to be reloaded, otherwise <code>false</code>.
     */
    public boolean hasChanged()
    {
        return changed;
    }

    /**
     * Sets the flag to indicate that the persistence layer has changed and needs to be re-initialized (see
     * {@link #initialize()}).
     * 
     * @param changed
     *            <code>true</code> if the persistence layer needs to be reloaded, otherwise
     *            <code>false</code>
     */
    public void setChanged( boolean changed )
    {
        this.changed = changed;
    }

    // //////////////////////////////////////////////////////////////////////////////////
    // ////// method implementations ////////
    // //////////////////////////////////////////////////////////////////////////////////

    /**
     * @param configuration
     *            Configuration of the wsag4j engine.
     * 
     * @return Agreement factory prototype based on the wsag4j configuration.
     * 
     * @throws Exception
     *             failed to initialize the configured factory
     */
    protected IAgreementFactory getAgreementFactoryPrototype( WSAG4JEngineConfigurationType configuration )
        throws Exception
    {
        String implementationClass = null;

        try
        {
            implementationClass =
                configuration.getFactory().getFactoryImplementation().getImplementationClass();

            if ( implementationClass == null )
            {
                throw new Exception();
            }
        }
        catch ( Exception ex )
        {
            String message =
                "Error in WSAG4J configuration: "
                    + "Could not load agreement factory implementation from configuration file.";
            throw new Exception( message, ex );
        }

        try
        {

            Object instance = Class.forName( implementationClass ).newInstance();

            if ( instance instanceof AgreementFactory )
            {
                AgreementFactory impl = (AgreementFactory) instance;

                IAgreementFactory factory = null;

                //
                // if the implementation is not an instance of
                // IWSAG4JAgreementFactory, return a facade for
                // the implementation.
                //
                if ( impl instanceof IAgreementFactory )
                {
                    factory = (IAgreementFactory) impl;
                }
                else
                {
                    factory = new AgreementFactoryFacade( impl );
                }

                //
                // explicitly initialize the factory context
                // and set the engine configuration
                //
                factory.setFactoryContext( new AgreementFactoryContext( factory ) );
                factory.setEngineConfiguration( configuration );

                //
                // also populate the factory context with the factory
                // configuration, so that action implementations may
                // look up the factory configuration
                //
                factory.getFactoryContext().put( WsagConstants.WSAG4J_FACTORY_CONFIGURATION, configuration );

                return factory;

            }
            else
            {
                Object[] filler = new Object[] { implementationClass, IAgreementFactoryHome.class.getName() };
                String msgLoadPersistenceLayerError =
                    "Error loading WSAG4J persistence layer. Class [{0}] does not implement interface [{1}].";
                String message = MessageFormat.format( msgLoadPersistenceLayerError, filler );
                throw new Exception( message );
            }

        }
        catch ( ClassNotFoundException e )
        {
            String message =
                MessageFormat.format(
                    "Error loading WSAG4J persistence layer. Class [{0}] not found. Error: {1}",
                    implementationClass, e.getMessage() );
            throw new Exception( message );
        }
        catch ( InstantiationException e )
        {
            String message =
                MessageFormat.format( "Error loading WSAG4J persistence layer. "
                    + "Class [{0}] could not be instantiated. Error: {1}", implementationClass,
                    e.getMessage() );
            throw new Exception( message );
        }
        catch ( IllegalAccessException e )
        {
            String message =
                MessageFormat.format( "Error loading WSAG4J persistence layer. "
                    + "Class [{0}] could not be accessed. Error: {1}", implementationClass, e.getMessage() );
            throw new Exception( message );
        }

    }

    /**
     * {@inheritDoc}
     */
    public void initialize() throws Exception
    {
        synchronized ( this )
        {

            try
            {
                persistentFactories.clear();
                factoriesById.clear();

                PersistentAgreementFactory[] factories = doLoad();

                for ( int i = 0; i < factories.length; i++ )
                {
                    persistentFactories.add( factories[i] );
                    factoriesById.put( factories[i].getResourceId(), factories[i] );
                }

                setChanged( false );
            }
            catch ( Exception e )
            {
                Object[] filler = new Object[] { e.getMessage() };
                String message =
                    MessageFormat.format(
                        "Could not load WSAG4J factory instance. Ignoring this instance. Error: {0}", filler );
                LOG.error( message );
            }

        }
    }

    /**
     * Lists the agreement for the factory with the given id.
     * 
     * @param agreementFactoryId
     *            the id of the factory for which the agreements are listed
     * 
     * @return the factory with the given id
     * 
     * @throws Exception
     *             indicates an error while looking up the factory at the persistence layer
     */
    public PersistentAgreementFactory find( String agreementFactoryId ) throws Exception
    {
        if ( hasChanged() )
        {
            initialize();
        }

        PersistentAgreementFactory persistentAgreementFactory = null;

        // check if the factory is known/available
        if ( factoriesById.containsKey( agreementFactoryId ) )
        {
            persistentAgreementFactory = factoriesById.get( agreementFactoryId );
        }
        else
        {
            throw new Exception( MessageFormat.format( "No agreement factory with id ''{0}'' found.",
                agreementFactoryId ) );
        }

        return persistentAgreementFactory;
    }

    /**
     * {@inheritDoc}
     */
    public PersistentAgreementFactory[] list() throws Exception
    {

        if ( hasChanged() )
        {
            initialize();
        }

        return persistentFactories.toArray( new PersistentAgreementFactory[persistentFactories.size()] );
    }

    /**
     * {@inheritDoc}
     */
    public void remove( String factoryId ) throws Exception
    {

        if ( hasChanged() )
        {
            initialize();
        }

        if ( factoriesById.containsKey( factoryId ) )
        {
            PersistentAgreementFactory factory = factoriesById.get( factoryId );

            if ( doRemove( factory ) )
            {
                persistentFactories.remove( factory );
                factoriesById.remove( factoryId );
            }
        }
    }
}
