/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.server.persistence.impl;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.ogf.graap.wsag.api.logging.LogMessage;
import org.ogf.graap.wsag.server.persistence.IAgreementFactoryHome;
import org.ogf.graap.wsag.server.persistence.PersistentAgreementFactory;
import org.ogf.graap.wsag4j.types.configuration.WSAG4JEngineConfigurationType;

/**
 * This class serves as a facade for multiple wsag4j engine instances that are deployed in a server. Since
 * multiple multiple wsag4j engine instances can be deployed in a server this facade allows to list, save and
 * remove agreement factories via the {@link IAgreementFactoryHome} interface as there would be only one
 * engine instance.
 * 
 * @author owaeld
 */
public class WSAG4JPersistenceFacade implements IAgreementFactoryHome
{

    private static final Logger LOG = Logger.getLogger( WSAG4JPersistenceFacade.class );

    private WSAG4JEngineConfigurationType[] config = new WSAG4JEngineConfigurationType[] { null };;

    /**
     * A map that contains the available persistent agreement factory which are identified by their factory
     * id.
     */
    protected Map<String, PersistentAgreementFactory> persistentFactories =
        new HashMap<String, PersistentAgreementFactory>();

    /**
     * An ordered list with the available agreement factories. The factories are stored in the order they were
     * retrieved from the persistence layer.
     */
    protected List<PersistentAgreementFactory> factoriesOL = new Vector<PersistentAgreementFactory>();

    /**
     * Creates a new facade for the given engine configurations.
     * 
     * @param config
     *            the configurations of the available WSAG4J engines
     */
    public WSAG4JPersistenceFacade( WSAG4JEngineConfigurationType[] config )
    {
        this.config = config;
    }

    /**
     * @see IAgreementFactoryHome#initialize()
     */
    public void initialize()
    {
        persistentFactories.clear();
        factoriesOL.clear();

        for ( int i = 0; i < config.length; i++ )
        {
            try
            {
                IAgreementFactoryHome home = loadPersistenceLayer( config[i] );
                home.setEngineConfiguration( config[i] );
                home.initialize();

                PersistentAgreementFactory[] factories = home.list();
                for ( int j = 0; j < factories.length; j++ )
                {
                    if ( persistentFactories.containsKey( factories[j].getResourceId() ) )
                    {
                        String message1 =
                            "[duplicated resource id] "
                                + "the agreement factory resource id must be unique in a WSAG4J engine.";
                        LOG.error( message1 );

                        String message2 =
                            "[duplicated resource id] the factory with resource id ''{0}'' was not loaded.";
                        LOG.error( MessageFormat.format( message2,
                                                         new Object[] { factories[j].getResourceId() } ) );
                    }
                    else
                    {
                        persistentFactories.put( factories[j].getResourceId(), factories[j] );
                        factoriesOL.add( factories[j] );
                    }
                }
            }
            catch ( Exception e )
            {
                LOG.error( "error loading persistence layer", e );
            }
        }
    }

    private IAgreementFactoryHome loadPersistenceLayer( WSAG4JEngineConfigurationType configuration )
        throws Exception
    {
        String implementationClass =
            configuration.getFactory().getPersistenceImplementation().getImplementationClass();

        if ( LOG.isDebugEnabled() )
        {
            LOG.debug( LogMessage.getMessage( "Create ''{0}'' instance as persistence layer.",
                implementationClass ) );
        }

        try
        {
            Object instance = Class.forName( implementationClass ).newInstance();

            if ( instance instanceof IAgreementFactoryHome )
            {
                if ( LOG.isDebugEnabled() )
                {
                    LOG.debug( "Persistence layer instance created and loaded." );
                }

                return (IAgreementFactoryHome) instance;
            }
            else
            {
                throw new Exception(
                    MessageFormat.format(
                        "Error loading WSAG4J persistence layer. Class [{0}] does not implement interface [{1}].",
                        implementationClass, IAgreementFactoryHome.class.getName() ) );
            }
        }
        catch ( ClassNotFoundException e )
        {
            throw new Exception( MessageFormat.format(
                "Error loading WSAG4J persistence layer. Class [{0}] not found. Error: {1}",
                implementationClass, e.getMessage() ) );
        }
        catch ( InstantiationException e )
        {
            throw new Exception( MessageFormat.format(
                "Error loading WSAG4J persistence layer. Class [{0}] could not be instantiated. Error: {1}",
                implementationClass, e.getMessage() ) );
        }
        catch ( IllegalAccessException e )
        {
            throw new Exception( MessageFormat.format(
                "Error loading WSAG4J persistence layer. Class [{0}] could not be accessed. Error: {1}",
                implementationClass, e.getMessage() ) );
        }
    }

    /**
     * looks up a factory with the given id.
     * 
     * @param factoryId
     *            the id of the factory to find
     * 
     * @return the factory
     * 
     * @throws Exception
     *             an error occurred while looking up the factory.
     * 
     * @see IAgreementFactoryHome#find(String)
     */
    public PersistentAgreementFactory find( String factoryId ) throws Exception
    {
        if ( persistentFactories.containsKey( factoryId ) )
        {
            return persistentFactories.get( factoryId );
        }

        return null;
    }

    /**
     * @return all available persistent factories
     * 
     * @throws Exception
     *             indicates an error while generating the factory list
     * 
     * @see org.ogf.graap.wsag.server.persistence.IAgreementFactoryHome#list()
     */
    public PersistentAgreementFactory[] list() throws Exception
    {
        return factoriesOL.toArray( new PersistentAgreementFactory[persistentFactories.size()] );
    }

    /**
     * @param factoryId
     *            removes the factory with the given id from the persistence layer
     * 
     * @throws Exception
     *             an error occurred while removing the factory
     * 
     * @see org.ogf.graap.wsag.server.persistence.IAgreementFactoryHome#remove(java.lang.String)
     */
    public void remove( String factoryId ) throws Exception
    {
        if ( persistentFactories.containsKey( factoryId ) )
        {
            factoriesOL.remove( persistentFactories.get( factoryId ) );
            persistentFactories.remove( factoryId );
        }
    }

    /**
     * Saves all factories.
     * 
     * @throws Exception
     *             indicates an error while saving the factories
     */
    public void save() throws Exception
    {
        PersistentAgreementFactory[] factories = list();
        for ( int i = 0; i < factories.length; i++ )
        {
            factories[i].save();
        }
    }

    /**
     * @param configs
     *            the engine configuration to set
     * 
     * @see org.ogf.graap.wsag.server.api.IEngineComponent#setEngineConfiguration(org.ogf.graap.wsag4j.types.configuration.WSAG4JEngineConfigurationType)
     */
    public void setEngineConfiguration( WSAG4JEngineConfigurationType configs )
    {
        this.config = new WSAG4JEngineConfigurationType[] { configs };
    }

    /**
     * @return the engine configuration
     * @see org.ogf.graap.wsag.server.api.IEngineComponent#getEngineConfiguration()
     */
    public WSAG4JEngineConfigurationType getEngineConfiguration()
    {
        return config[0];
    }

    /**
     * @param factories
     *            the factories to save
     * 
     * @throws Exception
     *             indicates an error while saving the factories
     * 
     * @see org.ogf.graap.wsag.server.persistence.IAgreementFactoryHome#saveAgreementFactories(org.ogf.graap.wsag.server.persistence.PersistentAgreementFactory[])
     */
    public void saveAgreementFactories( PersistentAgreementFactory[] factories ) throws Exception
    {
        for ( int i = 0; i < factories.length; i++ )
        {
            factories[i].save();
        }
    }

}
